<?php

namespace Pg\Modules\File_uploads\Models;

/**
 * File uploads module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('FILE_UPLOADS_TABLE')) {
    define('FILE_UPLOADS_TABLE', DB_PREFIX . 'file_uploads');
}

/**
 * File uploads config model
 *
 * @package 	PG_RealEstate
 * @subpackage 	File uploads
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class File_uploads_config_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Properties of file upload object in data source
     *
     * @var array
     */
    public $fields_all = array(
        'id',
        'gid',
        'name',
        'max_size',
        'name_format',
        'file_formats',
        'date_add',
    );

    /**
     * Available file formats of connfiguration
     *
     * @var array
     */
    public $file_formats = array();

    /**
     * File categories
     *
     * @var array
     */
    public $file_categories = array();

    /**
     * Constructor
     *
     * @return File_uploads_config_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();

        if (count($this->file_categories) == 0) {
            if (@require(APPPATH . 'config/file_uploads_mimes' . EXT)) {
                $this->file_categories = $mimes_categories;
            }
        }

        if (count($this->file_formats) == 0) {
            $this->ci->load->library('upload');
            $this->ci->upload->mimes_types('');
            $this->file_formats = $this->ci->upload->mimes;
        }
    }

    /**
     * Return configuration objects as array
     *
     * @return array
     */
    public function get_config_list()
    {
        $data = array();
        $this->ci->db->select(implode(", ", $this->fields_all))->from(FILE_UPLOADS_TABLE)->order_by('gid ASC');
        $results = $this->ci->db->get()->result_array();

        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[] = $this->format_config($r);
            }
        }

        return $data;
    }

    /**
     * Return configuration object by identifier
     *
     * @param integer $config_id configuration identifier
     *
     * @return array
     */
    public function get_config_by_id($config_id)
    {
        $data = array();
        $result = $this->ci->db->select(implode(", ", $this->fields_all))->from(FILE_UPLOADS_TABLE)->where("id",
                                                                                                           $config_id)->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }

        return $data;
    }

    /**
     * Return file upload configuration object by guid
     *
     * @param string $config_gid configuration guid
     *
     * @return array
     */
    public function get_config_by_gid($config_gid)
    {
        $data = array();
        $result = $this->ci->db->select(implode(", ", $this->fields_all))->from(FILE_UPLOADS_TABLE)->where("gid",
                                                                                                           $config_gid)->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }

        return $data;
    }

    /**
     * Save configuration object to data source
     *
     * @param integer $config_id configuration identifier
     * @param array   $data      configuration data
     *
     * @return integer
     */
    public function save_config($config_id, $data)
    {
        if (is_null($config_id)) {
            $data["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(FILE_UPLOADS_TABLE, $data);
            $config_id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $config_id);
            $this->ci->db->update(FILE_UPLOADS_TABLE, $data);
        }

        return $config_id;
    }

    /**
     * Remove configuration object by identifier
     *
     * @param string $config_id configuration identifier
     *
     * @return void
     */
    public function delete_config($config_id)
    {
        $data = $this->get_config_by_id($config_id);
        if (empty($data)) {
            return false;
        }

        $data = $this->format_config($data);

        $this->ci->db->where('id', $config_id);
        $this->ci->db->delete(FILE_UPLOADS_TABLE);

        return;
    }

    /**
     * Format configuration object
     *
     * @param array $data configuration data
     *
     * @return array
     */
    public function format_config($data)
    {
        if (!empty($data["file_formats"])) {
            $data["file_formats"] = unserialize($data["file_formats"]);
            if (!empty($data["file_formats"])) {
                foreach ($data["file_formats"] as $format) {
                    $data["enable_formats"][$format] = 1;
                }
                $data["file_formats_str"] = implode("|", $data["file_formats"]);
            }
        } else {
            $data["file_formats_str"] = "";
        }

        return $data;
    }

    /**
     * Validate configuration object for saving to data source
     *
     * @param integer $config_id configuration identifier
     * @param array   $data      configuration data
     *
     * @return array
     */
    public function validate_config($config_id = null, $data = array())
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["name"])) {
            $return["data"]["name"] = strip_tags($data["name"]);
            if (empty($return["data"]["name"])) {
                $return["errors"][] = l('error_title_invalid', 'file_uploads');
            }
        }

        if (isset($data["gid"])) {
            $return["data"]["gid"] = strip_tags($data["gid"]);
            $return["data"]["gid"] = preg_replace("/[\n\t\s]{1,}/", "-", trim($return["data"]["gid"]));
            if (empty($return["data"]["gid"])) {
                $return["errors"][] = l('error_gid_invalid', 'file_uploads');
            }
        } elseif (!$config_id) {
            $return["errors"][] = l('error_gid_invalid', 'file_uploads');
        }

        if (isset($data["max_size"])) {
            $return["data"]["max_size"] = intval($data["max_size"]);
        }

        if (isset($data["name_format"])) {
            $return["data"]["name_format"] = $data["name_format"];
        }

        if (isset($data["file_formats"])) {
            if (!is_array($data["file_formats"]) || empty($data["file_formats"])) {
                $return["errors"][] = l('error_empty_file_formats', 'file_uploads');
            } else {
                $return["data"]["file_formats"] = serialize($data["file_formats"]);
            }
        }

        return $return;
    }

}
