<?php

namespace Pg\Modules\Social_networking\Models;

/**
 * Social networking module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('SOCIAL_NETWORKING_PAGES_TABLE')) {
    define('SOCIAL_NETWORKING_PAGES_TABLE', DB_PREFIX . 'social_networking_pages');
}

/**
 * Social networking pages model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Social networking
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Social_networking_pages_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Social page object properties in data source
     *
     * @var array
     */
    public $fields_all = array(
        'id',
        'controller',
        'method',
        'name',
        'data',
    );

    /**
     * Module enabled
     *
     * @var boolean
     */
    public $enabled = true;

    /**
     * Temporary pages of social widgets
     *
     * @var array
     */
    public $temp_pages_list = array();

    /**
     * Pages cache  of social widgets
     *
     * @var array
     */
    protected $page_cache = array();

    /**
     * Class constructor
     *
     * @return Social_networking_pages_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return social pages objects from data source as array
     *
     * @param array $order_by          sorting data
     * @param array $params            sql criteria of query to data source
     * @param array $filter_object_ids filters identifiers
     *
     * @return array
     */
    public function get_pages_list($order_by = null, $params = array(), $filter_object_ids = null)
    {
        $data = array();
        $select_attrs = $this->fields_all;

        $this->ci->db->select(implode(", ", $select_attrs))->from(SOCIAL_NETWORKING_PAGES_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->fields_all)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }

        $results = $this->ci->db->get()->result_array();

        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[$r['id']] = $this->format_page($r);
            }
        }

        return $data;
    }

    /**
     * Return social page object from data source by identifier
     *
     * @param integer $page_id page identifier
     *
     * @return array
     */
    public function get_page_by_id($page_id)
    {
        $data = array();
        $result = $this->ci->db->select(implode(", ", $this->fields_all))
                        ->from(SOCIAL_NETWORKING_PAGES_TABLE)->where("id", $page_id)
                        ->get()->result_array();
        if (!empty($result)) {
            $data = $this->format_page($result[0]);
        }

        return $data;
    }

    /**
     * Format page object data
     *
     * @param array $data page data
     *
     * @return array
     */
    public function format_page($data = array())
    {
        $data['data'] = @unserialize($data['data']);

        return $data;
    }

    /**
     * Validate page object for saving to data source
     *
     * @param integer $page_id page identifier
     * @param array   $data    page data
     *
     * @return array
     */
    public function validate_page($page_id = false, $data = array())
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["controller"])) {
            $return["data"]["controller"] = strip_tags($data["controller"]);
        }

        if (isset($data["method"])) {
            $return["data"]["method"] = strip_tags($data["method"]);
        }

        if (isset($data["name"])) {
            $return["data"]["name"] = strip_tags($data["name"]);
        }

        if (isset($data["data"])) {
            $return["data"]["data"] = @serialize($data["data"]);
        }

        return $return;
    }

    /**
     * Save page object to data source by identifier
     *
     * @param integer $page_id page identifier
     * @param array   $data    page data
     *
     * @return integer
     */
    public function save_page($page_id = false, $data = array())
    {
        $data = (array) $data;
        if (is_null($page_id)) {
            $this->ci->db->insert(SOCIAL_NETWORKING_PAGES_TABLE, $data);
            $page_id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $page_id);
            $this->ci->db->update(SOCIAL_NETWORKING_PAGES_TABLE, $data);
        }

        return $page_id;
    }

    /**
     * Remove page object from data source by identifier
     *
     * @param integer $page_id page identifier
     *
     * @return void
     */
    public function delete_page($page_id = false)
    {
        $this->ci->db->where('id', $page_id);
        $this->ci->db->delete(SOCIAL_NETWORKING_PAGES_TABLE);

        return;
    }

    /**
     * Remove pages objects from data source by controller name
     *
     * @param string $controller controller name
     *
     * @return void
     */
    public function delete_pages_by_controller($controller = '')
    {
        $this->ci->db->where('controller', $controller);
        $this->ci->db->delete(SOCIAL_NETWORKING_PAGES_TABLE);

        return;
    }

    /**
     * Return page data by controller and method
     *
     * @param string $controller controller name
     * @param string $method     method name
     *
     * @return array
     */
    public function get_page_data($controller, $method)
    {
        if (isset($this->page_cache[$controller . '/' . $method])) {
            return $this->page_cache[$controller . '/' . $method];
        }
        $params = array('where' => array('controller' => $controller, 'method' => $method));
        $results = $this->get_pages_list(array('id' => 'desc'), $params);
        if (is_array($results) && count($results) > 0) {
            $this->page_cache[$controller . '/' . $method] = current($results);

            return $this->page_cache[$controller . '/' . $method];
        }

        return array();
    }

}
