<?php

namespace Pg\Modules\Users_deactivate\Models;

/**
 * Users deactivate module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
define("DEACTIVATED_ALERTS_TABLE", DB_PREFIX . "user_deactivated_alerts");

/**
 * Deactivated alert Model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users deactivate
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_deactivated_alert_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Deactivated user object properties in data source
     *
     * @var array
     */
    protected $fields = array(
        "id",
        "fname",
        "sname",
        "name",
        "email",
        "phone",
        "id_reason",
        "message",
        "status",
        "date_add",
    );

    /**
     * Settings for formatting deactivated user data
     *
     * @var array
     */
    protected $format_settings = array(
        "use_format" => true,
        "get_reason" => false,
    );

    /**
     * Constructor
     *
     * return Users_deactivated_alert_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return deactivated alert object by identifier
     *
     * @param integer $id        deactivated alert identifier
     * @param boolean $formatted format results
     *
     * @return array/boolean
     */
    public function get_alert_by_id($id, $formatted = false)
    {
        $id = intval($id);

        $this->ci->db->select(implode(", ", $this->fields));
        $this->ci->db->from(DEACTIVATED_ALERTS_TABLE);
        $this->ci->db->where("id", $id);

        //_compile_select;
        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            $rt = get_object_vars($result[0]);
            if ($formatted) {
                $rt = $this->format_alert(array($rt));
            }

            return $rt[0];
        } else {
            return false;
        }
    }

    /**
     * Save deactivated alert object to data source
     *
     * @param integer $id   alert identifier
     * @param array   $data alert data for saving
     *
     * @return integer
     */
    public function save_alert($id, $data)
    {
        if (!$id) {
            $data["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(DEACTIVATED_ALERTS_TABLE, $data);
            $id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where("id", $id);
            $this->ci->db->update(DEACTIVATED_ALERTS_TABLE, $data);
        }

        return $id;
    }

    /**
     * Remove deactivated alert from data source
     *
     * @param integer $id alert identifier
     *
     * @return void
     */
    public function delete_alert($id)
    {
        $alert = $this->get_alert_by_id($id, true);
        if (!$alert) {
            return false;
        }

        $this->ci->db->where("id", $id);
        $this->ci->db->delete(DEACTIVATED_ALERTS_TABLE);
    }

    /**
     * Mark deactivated alert as read
     *
     * @param integer $id alert identifier
     *
     * @return void
     */
    public function mark_as_read($id)
    {
        $data["status"] = 1;
        $this->save_alert($id, $data);
    }

    /**
     * Return deactivated alerts object from data source as array
     *
     * @param integer $page     page of results
     * @param string  $limits   items per page
     * @param array   $order_by sorting data
     * @param array   $params   sql criteria of query to data source
     *
     * @return array
     */
    private function _get_alerts_list($page = null, $limits = null, $order_by = null, $params = array())
    {
        $this->ci->db->select(implode(", ", $this->fields));
        $this->ci->db->from(DEACTIVATED_ALERTS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->fields)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        } elseif ($order_by) {
            $this->ci->db->order_by($order_by);
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($limits, $limits * ($page - 1));
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[] = $r;
            }

            return $this->format_alert($data);
        }

        return array();
    }

    /**
     * Return number of deactivated alerts in data source
     *
     * @param array $params sql criteria of query to data source
     *
     * @return integer
     */
    protected function _get_alerts_count($params = null)
    {
        $this->ci->db->select("COUNT(*) AS cnt");
        $this->ci->db->from(DEACTIVATED_ALERTS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Return deactivated alerts objects from data source as array
     *
     * @param integer $page     page of results
     * @param string  $limits   items per page
     * @param array   $order_by sorting data
     *
     * @return array
     */
    public function get_alerts_list($page = null, $limits = null, $order_by = null)
    {
        return $this->_get_alerts_list($page, $limits, $order_by);
    }

    /**
     * Return number of deactivated alerts objects in data source
     *
     * @return integer
     */
    public function get_alerts_count()
    {
        return $this->_get_alerts_count();
    }

    /**
     * Validate deactivate alert object for saving to data source
     *
     * @param integer $id   alert identifier
     * @param array   $data alert data
     *
     * @return array
     */
    public function validate_alert($id, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["id"])) {
            $return["data"]["id"] = intval($data['id']);
        }

        if (isset($data["fname"])) {
            $return["data"]["fname"] = trim(strip_tags($data["fname"]));
        }

        if (isset($data["sname"])) {
            $return["data"]["sname"] = trim(strip_tags($data["sname"]));
        }

        if (isset($data["name"])) {
            $return["data"]["name"] = trim(strip_tags($data["name"]));
        }

        if (isset($data["email"])) {
            $return["data"]["email"] = trim(strip_tags($data["email"]));
        }

        if (isset($data["phone"])) {
            $return["data"]["phone"] = trim(strip_tags($data["phone"]));
        }

        if (isset($data["id_reason"])) {
            $return["data"]["id_reason"] = intval($data["id_reason"]);
            if (empty($return["data"]["id_reason"])) {
                $return["errors"][] = l("error_empty_deactivated_reason", "users_deactivate");
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_deactivated_reason", "users_deactivate");
        }

        if (isset($data["message"])) {
            $return["data"]["message"] = trim(strip_tags($data["message"]));
        }

        if (isset($data["status"])) {
            $return["data"]["status"] = intval($data['status']);
        }

        return $return;
    }

    /**
     * Validate deactivate settings for saving to data source
     *
     * @param array $data settings data
     *
     * @return array
     */
    public function validate_settings($data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["deactivated_email"])) {
            $return["data"]["deactivated_email"] = trim(strip_tags($data["deactivated_email"]));
            if (!empty($return["data"]["deactivated_email"])) {
                $this->ci->config->load("reg_exps", true);
                $email_expr = $this->ci->config->item("email", "reg_exps");
                $return["data"]["_deactivated_email"] = strip_tags($return["data"]["deactivated_email"]);
                if (empty($return["data"]["deactivated_email"]) || !preg_match($email_expr, $return["data"]["deactivated_email"])) {
                    $return["errors"][] = l("error_email_incorrect", "users_deactivate");
                }
            }
        }

        if (isset($data["deactivated_send_mail"])) {
            $return["data"]["deactivated_send_mail"] = $data["deactivated_send_mail"] ? 1 : 0;
        }

        if (isset($data["deactivated_send_alert"])) {
            $return["data"]["deactivated_send_alert"] = $data["deactivated_send_alert"] ? 1 : 0;
            if ($return["data"]["deactivated_send_alert"]) {
                if (isset($return["data"]["deactivated_email"])) {
                    if (empty($return["data"]["deactivated_email"])) {
                        $return["errors"][] = l("error_empty_deactivated_email", "users_deactivate");
                    }
                } else {
                    $deactivated_email = $this->ci->pg_module->get_module_config("users_deactivate", "deactivated_email");
                    if (empty($deactivated_email)) {
                        $return["errors"][] = l("error_empty_deactivated_email", "users_deactivate");
                    }
                }
            }
        }

        return $return;
    }

    /**
     * Change settings for formatting alerts
     *
     * @param array $data settings data
     *
     * @return void
     */
    public function set_format_settings($data)
    {
        foreach ($data as $key => $value) {
            if (isset($this->format_settings[$key])) {
                $this->format_settings[$key] = $value;
            }
        }
    }

    /**
     * Format deactivated alert objects
     *
     * @param array $data alerts data
     *
     * @return array
     */
    public function format_alert($data)
    {
        if (!$this->format_settings["use_format"]) {
            return $data;
        }

        $reasons_search = array();

        foreach ($data as $key => $alert) {
            $data[$key] = $alert;
            //get reason
            if ($this->format_settings["get_reason"] && $alert["id_reason"]) {
                $reasons_search[] = $alert["id_reason"];
            }
            $data[$key]["message"] = nl2br($alert["message"]);
        }

        if ($this->format_settings["get_reason"] && !empty($reasons_search)) {
            $this->ci->load->model("users_deactivate/models/Users_deactivated_reason_model");

            $lang_id = $this->ci->pg_language->current_lang_id;
            $reference = $this->ci->pg_language->ds->get_reference($this->ci->Users_deactivated_reason_model->module_gid, $this->ci->Users_deactivated_reason_model->content[0], $lang_id);
            foreach ($data as $key => $alert) {
                $data[$key]["reason"] = isset($reference["option"][$alert["id_reason"]]) ?
                    $reference["option"][$alert["id_reason"]] : "";
            }
        }

        return $data;
    }

}
